@@ -427,7 +427,7 @@ for the hard coded one here.
 
 =head1 AUTHOR
 
-Phil Crow <philcrow2000@yahoo.com>
+Phil Crow <crow.phil@gmail.com>
 
 =head1 COPYRIGHT and LICENSE
 
@@ -476,10 +476,13 @@ BEGIN {
                 row_options
                 title
                 html_template
+                limit_by
                 all_fields_but
                 fields
                 extra_keys
                 form_name
+                expects
+                returns
             )
         )
     );
@@ -499,6 +502,7 @@ BEGIN {
                 html_form_options
                 html_form_foreign
                 date_select_text
+                html_form_raw_html
             )
         )
     );
@@ -582,13 +586,7 @@ sub do_main {
     $self->stash->view->template( 'main.tt' );
     $self->stash->view->title( '[% dist_name %]' );
 
-    $self->stash->view->data( {
-        pages => [
-[% FOREACH page IN pages %]
-            { link => '[% page.link %]', label => '[% page.label %]' },
-[% END %]
-        ],
-    } );
+    $self->stash->view->data( { pages => $self->site_links() } );
 } # END do_main
 
 #-----------------------------------------------------------------
@@ -602,7 +600,7 @@ sub site_links {
 [% IF page.link.match( '^/' ) %]
         { link => '[% page.link %]', label => '[% page.label %]' },
 [% ELSE %]
-        { link => $self->app_rootp() . '[% page.link %]', label => '[% page.label %]' },
+        { link => $self->app_rootp() . '/[% page.link %]', label => '[% page.label %]' },
 [% END %]
 [% END %]
     ];
@@ -930,17 +928,13 @@ at your option, any later version of Perl 5 you may have available.
 
 In [% package_name %]:
 
-    use [% gen_package_name %] qw(
-[% FOREACH method IN gen_methods %]
-        [% method +%]
-[% END %]
-    );
+    use base '[% gen_package_name %]';
 
 =head1 DESCRIPTION
 
 This module was generated by bigtop and IS subject to regeneration.
 Use it in [% package_name %] to provide the methods below.
-They are exported by default.
+Feel free to override them.
 
 =head1 METHODS
 
@@ -967,10 +961,19 @@ package [% gen_package_name %];
 
 use strict;
 
+[% IF wsdl %]
+use [% app_name %] qw(
+    -PluginNamespace=[% package_name +%]
+    SOAPMP20
+);
+
+our @ISA = qw( [% app_name %] );
+[% ELSE %]
 use base '[% app_name %]';
+[% END %]
 
 [% child_output %]
-
+[% IF wsdl %][% wsdl %][% END %]
 
 1;
 
@@ -1113,13 +1116,7 @@ sub [% config %] {
 [% END %]
 
 [% BLOCK main_links %]
-    $self->stash->view->data( {
-        pages => [
-[% FOREACH page IN pages %]
-            { link => '[% page.link %]', label => '[% page.label %]' },
-[% END %]
-        ],
-    } );
+    $self->stash->view->data( { pages => $self->site_links() } );
 [% END %]
 
 [% BLOCK site_links %]
@@ -1153,7 +1150,26 @@ sub [% config %] {
 [% END %]
 
 [% BLOCK main_table %]
-[%- IF dbix AND rows -%]
+[%- IF dbix AND rows AND limit_by -%]
+    my $query   = $self->get_arg_hash;
+    my $page    = $query->{ page } || 1;
+
+    my $schema  = $self->get_schema();
+    my $where   = ( $[% limit_by %] ) ? { [% limit_by %] => $[% limit_by %] } : undef;
+    my $results = $[% model %]->get_listing(
+        {
+            schema => $schema,
+            rows   => [% rows %],
+            page   => $page,
+            where  => $where,
+        }
+    );
+
+    $retval->{ page } = $results->pager();
+    my $rows          = $results->page();
+
+    while ( my $row = $rows->next ) {
+[%- ELSIF dbix AND rows -%]
     my $query   = $self->get_arg_hash;
     my $page    = $query->{ page } || 1;
 
@@ -1170,6 +1186,17 @@ sub [% config %] {
     my $rows          = $results->page();
 
     while ( my $row = $rows->next ) {
+[%- ELSIF dbix AND limit_by -%]
+    my $schema = $self->get_schema();
+    my $where  = ( $[% limit_by %] ) ? { [% limit_by %] => $[% limit_by %] } : undef;
+    my @rows   = $[% model %]->get_listing(
+        {
+            schema => $schema,
+            where  => $where,
+        }
+    );
+
+    foreach my $row ( @rows ) {
 [%- ELSIF dbix -%]
     my $schema = $self->get_schema();
     my @rows   = $[% model %]->get_listing( { schema => $schema } );
@@ -1335,6 +1362,65 @@ sub [% crud_name %]_edit {
     # remember to commit
 }
 [% END %]
+
+[% BLOCK SOAP_gen_method_body %]
+    my $self        = shift;
+    my $input       = $self->soap_in;
+    my $output_data = $self->[% internal_method %]( $input );
+
+    $self->template_disable( 1 );
+
+    return $self->soap_out( $output_data );
+[% END %]
+
+[% BLOCK SOAP_stub_method %]
+#-----------------------------------------------------------------
+# $self->[% internal_method %](  )
+#-----------------------------------------------------------------
+sub [% internal_method %] {
+    my ( $self, $input ) = @_;
+} # END [% internal_method %]
+[% END %]
+
+[% BLOCK soap_methods %]
+
+#-----------------------------------------------------------------
+# $self->namespace(  )
+#-----------------------------------------------------------------
+sub namespace {
+    return '[% stub_module %]';
+} # END namespace
+
+#-----------------------------------------------------------------
+# $self->get_soap_ops
+#-----------------------------------------------------------------
+sub get_soap_ops {
+    my $self = shift;
+
+    return {
+        soap_name      => '[% soap_name %]',
+        location       => $self->location,
+        namespace_base => '[% namespace_base %]',
+        operations     => [
+[% FOREACH op IN operations %]
+            {
+                name => '[% op.name %]',
+                expects => [
+[% FOREACH param IN op.expects %]
+                    { name => '[% param.name %]', type => '[% param.type %]' },
+[% END %]
+                ],
+                returns => [
+[% FOREACH param IN op.returns %]
+                    { name => '[% param.name %]', type => '[% param.type %]' },
+[% END %]
+                ],
+            },
+[% END %]
+        ],
+    };
+} # END get_description
+[% END %]
 EO_TT_blocks
 
 #-----------------------------------------------------------------
@@ -1669,7 +1755,7 @@ sub gen_Control {
             $saw_root++ if $var eq 'root';
         }
         unshift @configs, [ 'dbconn', 'dbi:SQLite:dbname=app.db' ];
-        push @configs, [ 'root', 'html' ] unless $saw_root;
+        push @configs, [ 'root', 'html/templates' ] unless $saw_root;
 
         # ...then, the locations
         my $locations = $bigtop_tree->walk_postorder(
@@ -1805,6 +1891,20 @@ sub output_test_locations {
     return \@retval;
 }
 
+package # join_table
+    join_table;
+use strict; use warnings;
+
+sub output_field_names {
+    my $self         = shift;
+    my $child_output = shift;
+    my $data         = shift;
+
+    return unless $self->{__NAME__} eq $data->{table_of_interest};
+
+    return $child_output;
+}
+
 package # table_block
     table_block;
 use strict; use warnings;
@@ -1844,6 +1944,9 @@ my %magical_uses = (
     AutoCRUD => 'Gantry::Plugins::AutoCRUD',
     stub     => '',
 );
+my %magical_gen_uses = (
+    SOAP     => 'Gantry::Plugins::SOAPMP20',
+);
 
 sub get_package_name {
     my $self = shift;
@@ -1885,7 +1988,7 @@ sub skip_base_controller {
 
 sub output_extra_use {
     my $self   = shift;
-    my $type   = $self->get_controller_type;
+    my $type   = shift;
     my $module = $magical_uses{ $type } || return;
 
     my $poser  = {
@@ -1900,6 +2003,23 @@ sub output_extra_use {
     return ( $output, $module );
 }
 
+sub output_extra_gen_use {
+    my $self   = shift;
+    my $type   = shift;
+    my $module = $magical_gen_uses{ $type } || return;
+
+    my $poser  = {
+        __ARGS__ => [ $module ]
+    };
+    bless $poser, 'controller_statement';
+
+    my %extra_use = @{ $poser->uses };
+
+    my $output    = $extra_use{ uses_output };
+
+    return ( $output, $module );
+}
+
 sub output_controllers {
     my $self         = shift;
     shift;
@@ -1929,8 +2049,12 @@ sub output_controllers {
     return if ( defined $statements->{no_gen} and $statements->{no_gen}[0] );
 
     # Begin by inserting magical things based on controller type
+    my $controller_type = $self->get_controller_type();
     my ( $extra_use, $extra_module )
-            = $self->output_extra_use( $self->get_controller_type() );
+            = $self->output_extra_use( $controller_type );
+
+    my ( $gen_extra_use, $gen_extra_module )
+            = $self->output_extra_gen_use( $controller_type );
 
     #############################################
     # Deal with what the children made for us.  #
@@ -1947,7 +2071,11 @@ sub output_controllers {
     my @depend_head = ( $data->{app_name} )
             unless ( $self->is_base_controller );
 
-    push @depend_head, $gen_package_name if defined $gen_method_names;
+    push @depend_head, $gen_package_name
+            if ( defined $gen_method_names
+                    or
+                 defined $output_hash->{ extra_stub_method_name }
+            );
 
     unshift @{ $output_hash->{used_modules} }, \@depend_head;
 
@@ -1959,6 +2087,34 @@ sub output_controllers {
         $output_str       = "\n$extra_use" . $output_str;
     }
 
+    if ( $gen_extra_use ) {
+        push @{ $used_modules }, $gen_extra_module;
+        chomp $gen_extra_use;
+        $gen_output_str   = "\n$gen_extra_use" . $gen_output_str;
+    }
+
+    # deal with SOAP stubs
+    if ( defined $output_hash->{ extra_stub_method_name } ) {
+        push @{ $stub_method_names },
+             @{ $output_hash->{ extra_stub_method_name } };
+    }
+
+    # ... and SOAP wsdl method
+    my $wsdl;
+    if ( defined $output_hash->{ soap_params } ) {
+        $wsdl = Bigtop::Backend::Control::Gantry::soap_methods(
+            {
+                operations     => $output_hash->{ soap_params },
+                soap_name      => $statements->{ soap_name }[0],
+                namespace_base => $statements->{ namespace_base }[0],
+                stub_module    => $package_name,
+            }
+        );
+        if ( $wsdl ) {
+            push @{ $gen_method_names }, qw( namespace get_soap_ops );
+        }
+    }
+
     # make doc stubs for standard controller accessor methods
     if ( defined $statements->{controls_table} ) {
         push @{ $stub_method_names }, qw( get_model_name text_descr );
@@ -2103,12 +2259,9 @@ sub output_controllers {
             {
                 child_output       => $gen_output_str,
                 gen_package_name   => $gen_package_name,
-#                external_modules   => \@external_modules,
                 init_sub           => $init_sub,
                 config_accessors   => $config_accessors,
                 gen_pod            => $gen_pod,
-#                full_use_statement => $full_use_statement,
-#                pages              => \@pages,
                 %{ $data },                # Go fish!
             }
         );
@@ -2144,9 +2297,11 @@ sub output_controllers {
             {
                 app_name         => $data->{app_name},
                 gen_package_name => $gen_package_name,
+                package_name     => $package_name,
                 child_output     => $gen_output_str,
                 export_array     => $export_array,
                 gen_pod          => $gen_pod,
+                wsdl             => $wsdl,
             }
         );
     }
@@ -2632,9 +2787,7 @@ sub output_controller {
                               {$method_name}
                               {statements};
 
-    if ( $statements->{no_gen} ) {
-        return;
-    }
+    return if ( $statements->{no_gen} );
 
     # restart recursion based on method type
     unless ( $method_body->can( "output_$type" ) ) {
@@ -2682,6 +2835,17 @@ sub output_controller {
         );
     }
 
+    if ( $child_output->{ extra_comment_methods } ) {
+        foreach my $method ( @{ $child_output->{ extra_comment_methods } } ) {
+            $output .= Bigtop::Backend::Control::Gantry::controller_method(
+                {
+                    method_name      => $method,
+                    gen_package_name => $gen_package_name,
+                }
+            );
+        }
+    }
+
     if ( $child_output->{stub_output} ) {
         $output = Bigtop::Backend::Control::Gantry::gen_controller_method(
             {
@@ -2691,6 +2855,13 @@ sub output_controller {
         );
     }
 
+    my $extra_stub_method;
+
+    if ( $child_output->{ extra_for_stub } ) {
+        $output .= "\n$child_output->{ extra_for_stub }{ full_sub }\n";
+        $extra_stub_method = $child_output->{ extra_for_stub }{ name };
+    }
+
     if ( $child_output->{crud_output} ) {
         my $crud_name    = $self->{__NAME__};
         $crud_name       =~ s/_form//;
@@ -2730,6 +2901,8 @@ sub output_controller {
             output           => $output,
             stub_method_name => $stub_method_name,
             gen_method_name  => $gen_method_name,
+            extra_stub_method_name => $extra_stub_method,
+            soap_params      => $child_output->{ soap_params },
         ]
     ];
 }
@@ -2869,7 +3042,7 @@ sub output_links {
         }
         else {
             $abs_page = {
-                link => qq{\$self->app_rootp() . '$page->{ link }'},
+                link => qq{\$self->app_rootp() . '/$page->{ link }'},
             };
         }
         $abs_page->{ label } = $page->{ label };
@@ -2898,6 +3071,7 @@ sub output_main_listing {
     my $data         = shift;
 
     my $choices      = { @{ $child_output } };
+    my @optional_args;
 
     # see if we are paging
     my $rows = $choices->{ rows }[0] || undef;
@@ -2905,6 +3079,12 @@ sub output_main_listing {
         $rows = '$self->' . $choices->{ paged_conf }[0];
     }
 
+    # see if we are limiting output rows by foreign key
+    my $limit_by = $choices->{ limit_by }[0] || undef;
+    if ( defined $limit_by ) {
+        push @{ $choices->{ extra_args} }, '$' . $limit_by;
+    }
+
     # set up args
     my ( $arg_capture, @doc_args )
             = _build_arg_capture( @{ $choices->{extra_args} } );
@@ -2959,12 +3139,15 @@ sub output_main_listing {
             push @col_labels, { simple => $label };
         }
 
-        # see if it's foreigner
+        # see if it's foreigner or has a special display method
         if ( defined $fields->{$col}{refers_to} ) {
-            push @cols, ${col} . '->foreign_display()';
+            push @cols, $col . '->foreign_display()';
+        }
+        elsif ( defined $fields->{ $col }{ html_form_options } ) {
+            push @cols, "${col}_display()";
         }
         else {
-            push @cols, ${col};
+            push @cols, $col;
         }
     }
 
@@ -2991,6 +3174,7 @@ sub output_main_listing {
             data_cols   => \@cols,
             row_options => $row_options,
             dbix        => $self->is_dbix_class( $data ),
+            limit_by    => $limit_by,
         }
     );
 
@@ -3021,6 +3205,66 @@ sub is_dbix_class {
     return $config_block->{ dbix };
 }
 
+sub output_SOAP {
+    my $self         = shift;
+    my $child_output = shift;
+    my $data         = shift;
+    my $choices      = { @{ $child_output } };
+
+    my $extra_comment_methods;
+    if ( not defined $data->{ WSDL_COMMENTS } ) {
+        $extra_comment_methods = [ qw( namespace get_soap_ops ) ],
+
+        $data->{ WSDL_COMMENTS } = 'done';
+    }
+
+    my $handler_method  = $self->get_method_name();
+    ( my $internal_method = $handler_method ) =~ s/^do_//;
+
+    my $extra_sub = Bigtop::Backend::Control::Gantry::SOAP_stub_method(
+        {
+            handler_method  => $handler_method,
+            internal_method => $internal_method,
+        }
+    );
+
+    my %soap_params;
+    $soap_params{ name } = $internal_method;
+
+    foreach my $expected ( @{ $choices->{ expects } } ) {
+        if ( ref( $expected ) eq 'HASH' ) {
+            my ( $name, $type ) = %{ $expected };
+            push @{ $soap_params{ expects } },
+                  { name => $name, type => $type };
+        }
+        else {
+            push @{ $soap_params{ expects } },
+                  { name => $expected, type => 'xsd:string' };
+        }
+    }
+
+    foreach my $returned ( @{ $choices->{ returns } } ) {
+        if ( ref( $returned ) eq 'HASH' ) {
+            my ( $name, $type ) = %{ $returned };
+            push @{ $soap_params{ returns } },
+                  { name => $name, type => $type };
+        }
+        else {
+            push @{ $soap_params{ returns } },
+                  { name => $returned, type => 'xsd:string' };
+        }
+    }
+
+    return [
+        extra_for_stub => {
+            name     => $internal_method,
+            full_sub => $extra_sub,
+        },
+        extra_comment_methods => $extra_comment_methods,
+        soap_params => \%soap_params,
+    ];
+}
+
 # Given
 #   [ Label => url, Label2 => url2, Label_no_url; ]
 # Returns
@@ -3256,4 +3500,6 @@ sub output_base_links    { goto &walker_output; }
 
 sub output_links         { goto &walker_output; }
 
+sub output_SOAP          { goto &walker_output; }
+
 1;
