@@ -1433,6 +1433,6 @@ Here is the gist of the lookup hash in summary (as generated by outliner):
 
 =head1 AUTHOR
 
-Phil Crow <philcrow2000@yahoo.com>
+Phil Crow <crow.phil@gmail.com>
 
 =cut
